/**
 * Algorithme de Berlekamp pour la dcomposition des polynmes.
 */
public class Berlekamp {

    /**
     * Les polynmes irreductibles, resultat de la decomposition.
     */
    static private Polynome[] facteurs;

    static private int facteurIndex;

    Corps corps;

    int p;


    /**
     * Constructeur pour la dcomposition d'un polynome dans Fp.
     */
    public Berlekamp(int p, Polynome polynome) throws CorpsException {

	Polynome polyDebut = new Polynome(polynome);
	Main.outl("Dcomposition de " + polyDebut + " dans F" + p + 
		  " selon Berlekamp");

	// Vrifie si on peut "sortir" la caractristique
	boolean sortir = true;
	for (int i = 0; i <= polyDebut.degre(); i++) {
	    if ((polyDebut.coeff[i] != 0) && (i % p != 0)) sortir = false;
	}
	if (sortir) {
	    // Fait "sortir" la caracteristique
	    Polynome tmp = new Polynome(0);
	    Polynome poly1 = new Polynome(1);
	    for (int j = 0; j <= polyDebut.degre(); j++) {
		if ((j % p) == 0) { 
		    tmp = tmp.add(poly1.multKXn(polyDebut.coeff[j], 
						(j / p)));
		}
	    }
	    polyDebut = new Polynome(tmp);
	    Main.out("Le polynme a et simplifi en " + polyDebut);
	}

	// Cre un corps pour la dcomposition. m sera utilis pour 
	// engendrer tous les combinaisons linaires des lments de Fp, 
	// qui seront utilises pour la matrice B - Id; polyDebut
	// sera utilis pour simplifier les x^kp
	corps = new Corps(p, polyDebut);
	this.p = p;

	factorise(polyDebut);
    }



    /**
     * Initialise la structure de donnes pour les facteurs irrductibles.
     */
    public static void initialise() {
	facteurs = new Polynome[200];
	for (int i = 0; i < 200; i++)
	    facteurs[i] = new Polynome();
	facteurIndex = 0;
    }



    /**
     * Factorise poly.
     * Si poly n'est pas factorisable (irrductible), il est stock dans
     * l'array facteurs, sinon on factorise poly en deux polynomes et on
     * appelle rcursivement cette mthode sur chacun d'eux.
     */
    void factorise(Polynome poly) {

	Polynome[] gen = new Polynome[poly.degre()];

	// Engendre tous les x^0, x^p, x^2p, x^3p ... x^kp .
	// Les x^kp avec kp >= au degr de poly vont tre simplifis 
	Main.outl("Gnrateurs de la factorisation:");
	for (int k = 0; k < poly.degre(); k++) {
	    gen[k] = new Polynome(1).multKXn(1, k*p);
	    if (k*p >= poly.degre()) 
		gen[k] = corps.adjustePuissancesPolynome(gen[k]);
	    Main.out("x^" + (k*p) + " = " + gen[k]);
	}

	// Cre les matrices B et B-Id
	int[][] matB = new int[poly.degre()][poly.degre()]; 

	/*** TO DO ***/


    }




}
